/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const enrollmentProjectionApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const config = require(`${libs}/config`);

let rqf = new ReqQueryFields();

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValueToField({
    name: 'state',
    table: 'estado',
    tableField: ['nome', 'id'],
    resultField: ['state_name', 'state_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'projecao_matricula'
    }
}, 'dims').addValueToField({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'projecao_matricula'
    }
}, 'filter').addValueToField({
    name: 'city',
    table: 'municipio',
    tableField: ['nome', 'id'],
    resultField: ['city_name', 'city_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'projecao_matricula'
    }
}, 'dims').addValueToField({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'projecao_matricula'
    }
}, 'filter').addValue({
    name: 'min_year',
    table: 'projecao_matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'projecao_matricula',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
});

enrollmentProjectionApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.field("'Brasil'", 'name')
    .field('SUM(projecao_matricula.urbano_dia_total)', 'urban_day_total')
    .field('SUM(projecao_matricula.urbano_noite_total)', 'urban_night_total')
    .field('SUM(projecao_matricula.rural_dia_total)', 'rural_day_total')
    .field('SUM(projecao_matricula.rural_noite_total)', 'rural_night_total')
    .field('projecao_matricula.etapa_ensino_escola_ano_id', 'education_level_school_year_id')
    .field('projecao_matricula.ano_censo', 'year')
    .from('projecao_matricula')
    .where('projecao_matricula.etapa_ensino_escola_ano_id <> 7 AND projecao_matricula.etapa_ensino_escola_ano_id < 71')
    .group('projecao_matricula.etapa_ensino_escola_ano_id')
    .group('projecao_matricula.ano_censo')
    .order('projecao_matricula.ano_censo')
    .order('projecao_matricula.etapa_ensino_escola_ano_id');

    next();
}, query, id2str.transform(), (req, res, next) => {

    req.result.forEach((r) => {
        r.urban_day_total = parseInt(r.urban_day_total, 10);
        r.urban_night_total = ((r.education_level_school_year_id >= 3 && r.education_level_school_year_id < 10) || (r.education_level_school_year_id >= 30)) ? parseInt(r.urban_night_total, 10) : 0;   //Não conta matrículas noturnas da pré-escola e da creche
        r.rural_day_total = parseInt(r.rural_day_total, 10);
        r.rural_night_total = ((r.education_level_school_year_id >= 3 && r.education_level_school_year_id < 10) || (r.education_level_school_year_id >= 30)) ? parseInt(r.rural_night_total, 10) : 0;
    });
    
    next();
}, response('enrollment_projection'));

module.exports = enrollmentProjectionApp;